import { BaseConnection, WsConnection } from "tsrpc";
import { server } from "../..";
import { RandomEx } from "../../Engine/Utils/Number/RandomEx";
import { EGameState } from "../../shared/protocols/define/enum";
import { ServiceType } from "../../shared/protocols/serviceProto";
import { RoomConnData } from "./RoomUtils";


/**
 * Room
 */
export default class Room {

	//#region static

	private static maxRoomId: number = 0;
	private static rooms: { [roomId: number]: Room } = {};

	//#endregion

	//#region public

	public RoomId: number = 0;

	//#endregion

	//#region private

	private conns: BaseConnection[] = [];

	private connData = new Map<BaseConnection, RoomConnData>();

	private nowPlayer: [conn: BaseConnection, state: number] = undefined;

	//#endregion

	//#region Lifecycle

	/**
	 *
	 */
	constructor() {
		// 给每个新房间生成一个唯一的 ID
		this.RoomId = ++Room.maxRoomId;
		Room.rooms[this.RoomId] = this;
	}

	//#endregion

	//#region Custom

	/** GetRoom */
	public static GetRoom(roomId: number): Room {
		return this.rooms[roomId];
	}

	//#endregion

	//#region API

	/** Exit */
	public Exit(conn: BaseConnection): void {
		server.broadcastMsg("room/Exit", 0, <WsConnection<ServiceType>[]>this.conns);
	}

	/** GotoGame */
	public GotoGame(): void {
		server.broadcastMsg("room/GoToGame", 0, <WsConnection<ServiceType>[]>this.conns);
	}

	/** ChangeState */
	public ChangeState(conn: BaseConnection, state: EGameState): void {
		if (state === EGameState.Ready) {
			this.SetReady(conn);
		}
		this.connData.forEach((connData: RoomConnData, key: BaseConnection) => {
			const player: WsConnection<ServiceType>[] = <WsConnection<ServiceType>[]>this.conns.filter((conn) => conn === key);
			const apiNum: number = conn === key ? 100 : 200;
			server.broadcastMsg("room/ChangeState", [connData.state + apiNum], player);
		});
	}

	//#endregion

	//#region Custom

	/** Join */
	public Join(conn: BaseConnection): void {
		this.conns.push(conn);
		const data: RoomConnData = {
			state: undefined,
			card: undefined,
		};
		this.connData.set(conn, data);
	}

	/** SetReady */
	public SetReady(conn: BaseConnection): void {
		const self: this = this;
		const cardCount: number = 24;
		const data: RoomConnData = this.connData.get(conn);
		data.state = EGameState.Ready;
		this.connData.set(conn, data);
		let canStart: boolean = true;
		this.connData.forEach((connData: RoomConnData) => {
			if (connData.state === undefined || connData.state !== EGameState.Ready) {
				canStart = false;
			}
		});
		if (canStart) {
			const now: number = RandomEx.GetInt(0, 2);
			this.connData.forEach((connData: RoomConnData, key: BaseConnection) => {
				const card: number = RandomEx.GetInt(0, cardCount);
				connData.state = key === self.conns[now] ? EGameState.TurnCards : EGameState.Wait;
				connData.card = card;
			});
		}
	}

	/** RestartGame */
	public RestartGame(): void {

	}

	/** ConnCount */
	public ConnCount(): number {
		return this.conns.length;
	}

	//#endregion
}
